from __future__ import absolute_import, division, print_function

import pytest
import telnyx

CALL_CONTROL_ID = "AgDIxmoRX6QMuaIj_uXRXnPAXP0QlNfXczRrZvZakpWxBlpw48KyZQ=="


def create_dial():
    return telnyx.Call.create(
        connection_id="1111111111222222223", to="+12223334444", from_="+12223330000"
    )


class TestCall(object):
    def test_is_creatable(self, request_mock):
        resource = create_dial()
        request_mock.assert_requested("post", "/v2/calls")
        assert isinstance(resource, telnyx.Call)

    def test_can_call_reject(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.reject(cause="USER_BUSY")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/reject" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_reject(self, request_mock):
        resource = create_dial()
        resource.create_reject(CALL_CONTROL_ID, cause="USER_BUSY")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/reject" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_answer(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.answer()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/answer" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_answer(self, request_mock):
        resource = create_dial()
        resource.create_answer(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/answer" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_hangup(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.hangup()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/hangup" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_hangup(self, request_mock):
        resource = create_dial()
        resource.create_hangup(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/hangup" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_bridge(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.bridge(call_control_id=CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/bridge" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_bridge(self, request_mock):
        resource = create_dial()
        resource.create_bridge(CALL_CONTROL_ID, call_control_id=CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/bridge" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_fork_start(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.fork_start()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/fork_start" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_fork_start(self, request_mock):
        resource = create_dial()
        resource.create_fork_start(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/fork_start" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_fork_stop(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.fork_stop()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/fork_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_fork_stop(self, request_mock):
        resource = create_dial()
        resource.create_fork_stop(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/fork_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_gather_using_audio(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.gather_using_audio(audio_url="http://telnyx-audio.url")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/gather_using_audio" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_gather_using_audio(self, request_mock):
        resource = create_dial()
        resource.create_gather_using_audio(
            CALL_CONTROL_ID, audio_url="http://telnyx-audio.url"
        )
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/gather_using_audio" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_gather_using_speak(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.gather_using_speak(
            language="en-US", voice="female", payload="Hello from the other side"
        )
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/gather_using_speak" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_gather_using_speak(self, request_mock):
        resource = create_dial()
        resource.create_gather_using_speak(
            CALL_CONTROL_ID,
            language="en-US",
            voice="female",
            payload="Hello from the other side",
        )
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/gather_using_speak" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_gather_stop(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.gather_stop()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/gather_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_gather_stop(self, request_mock):
        resource = create_dial()
        resource.create_gather_stop(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/gather_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_playback_start(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.playback_start(audio_url="http://telnyx-audio.url")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/playback_start" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_playback_start(self, request_mock):
        resource = create_dial()
        resource.create_playback_start(
            CALL_CONTROL_ID, audio_url="http://telnyx-audio.url"
        )
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/playback_start" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_playback_stop(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.playback_stop()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/playback_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_playback_stop(self, request_mock):
        resource = create_dial()
        resource.create_playback_stop(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/playback_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_record_pause(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.record_pause()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/record_pause" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_record_pause(self, request_mock):
        resource = create_dial()
        resource.create_record_pause(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/record_pause" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_record_resume(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.record_resume()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/record_resume" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_record_resume(self, request_mock):
        resource = create_dial()
        resource.create_record_resume(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/record_resume" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_record_start(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.record_start(channels="single", format="mp3")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/record_start" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_record_start(self, request_mock):
        resource = create_dial()
        resource.create_record_start(CALL_CONTROL_ID, channels="single", format="mp3")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/record_start" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_record_stop(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.record_stop()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/record_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_record_stop(self, request_mock):
        resource = create_dial()
        resource.create_record_stop(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/record_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_refer(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.refer(sip_address="sip:username@sip.non-telnyx-address.com")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/refer" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_refer(self, request_mock):
        resource = create_dial()
        resource.create_refer(
            CALL_CONTROL_ID, sip_address="sip:username@sip.non-telnyx-address.com"
        )
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/refer" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_send_dtmf(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.send_dtmf(digits="1www2WABCDw9")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/send_dtmf" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_send_dtmf(self, request_mock):
        resource = create_dial()
        resource.create_send_dtmf(CALL_CONTROL_ID, digits="1www2WABCDw9")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/send_dtmf" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_speak(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.speak(
            language="en-US", voice="female", payload="Hello from the other side"
        )
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/speak" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_speak(self, request_mock):
        resource = create_dial()
        resource.create_speak(
            CALL_CONTROL_ID,
            language="en-US",
            voice="female",
            payload="Hello from the other side",
        )
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/speak" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_can_call_transcription_start(self, request_mock):
        resource = telnyx.Call()
        resource.call_control_id = CALL_CONTROL_ID
        resource.transcription_start(language="en")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/transcription_start" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_can_call_calls_transcription_start(self, request_mock):
        resource = telnyx.Call.create_transcription_start(
            CALL_CONTROL_ID, language="en"
        )
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/transcription_start" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.telnyx_object.TelnyxObject)

    def test_can_call_transcription_stop(self, request_mock):
        resource = telnyx.Call()
        resource.call_control_id = CALL_CONTROL_ID
        resource.transcription_stop()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/transcription_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_transcription_stop(self, request_mock):
        resource = telnyx.Call.create_transcription_stop(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/transcription_stop" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.telnyx_object.TelnyxObject)

    def test_can_call_transfer(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.transfer(to="+11111222222")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/transfer" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_transfer(self, request_mock):
        resource = create_dial()
        resource.create_transfer(CALL_CONTROL_ID, to="+11111222222")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/transfer" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_enqueue(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.enqueue("a_queue")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/enqueue" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_enqueue(self, request_mock):
        resource = create_dial()
        resource.create_enqueue(CALL_CONTROL_ID, queue_name="a_queue")
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/enqueue" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_leave_queue(self, request_mock):
        resource = create_dial()
        resource.call_control_id = CALL_CONTROL_ID
        resource.leave_queue()
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/leave_queue" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)

    def test_can_call_calls_leave_queue(self, request_mock):
        resource = create_dial()
        resource.create_leave_queue(CALL_CONTROL_ID)
        request_mock.assert_requested(
            "post", "/v2/calls/%s/actions/leave_queue" % CALL_CONTROL_ID
        )
        assert isinstance(resource, telnyx.Call)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "file_upload", path="actions/file_upload", operations=["create"]
)
class MessagingHostedNumber(
    CreateableAPIResource, DeletableAPIResource, ListableAPIResource
):
    OBJECT_NAME = "messaging_hosted_number"

    def file_upload(self, **params):
        return self.create_file_upload(**params)



from __future__ import absolute_import, division, print_function

import telnyx

TEST_RESOURCE_ID = "6a09cdc3-8948-47f0-aa62-74ac943d6c58"


class TestDetailRecordsReport(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.DetailRecordsReport.list()
        request_mock.assert_requested("get", "/v2/wireless/detail_records_reports")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.DetailRecordsReport)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.DetailRecordsReport.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/wireless/detail_records_reports/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.DetailRecordsReport)

    def test_is_creatable(self, request_mock):
        resource = telnyx.DetailRecordsReport.create(
            start_time="2018-02-02T22:25:27.521Z", end_time="2018-02-02T22:25:27.521Z"
        )
        request_mock.assert_requested("post", "/v2/wireless/detail_records_reports")
        assert isinstance(resource, telnyx.DetailRecordsReport)

    def test_is_deletable(self, request_mock):
        resource = telnyx.DetailRecordsReport.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/wireless/detail_records_reports/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import DeletableAPIResource, ListableAPIResource


class RoomRecording(DeletableAPIResource, ListableAPIResource):
    OBJECT_NAME = "room_recording"



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import APIResource


class NumberLookup(APIResource):
    OBJECT_NAME = "number_lookup"

    @classmethod
    def class_url(cls):
        return "/v2/number_lookup"



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods("token", path="token", operations=["create"])
class TelephonyCredential(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "telephony_credential"
    API_RECORD_TYPE_NAME = "credential"

    def token(self, **params):
        return TelephonyCredential.create_token(self.id, **params)



import telnyx

TEST_RESOURCE_ID = "12ade33a-21c0-473b-b055-b3c836e1c292"


class TestWebhookDeliveries(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.WebhookDeliveries.list()
        request_mock.assert_requested("get", "/v2/webhook_deliveries")
        assert isinstance(resources.data, list)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.WebhookDeliveries.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/webhook_deliveries/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.WebhookDeliveries)



from __future__ import absolute_import, division, print_function

import pytest
import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


class TestCallControlApplication(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.CallControlApplication.list()
        request_mock.assert_requested("get", "/v2/call_control_applications")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.CallControlApplication)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_retrievable(self, request_mock):
        resource = telnyx.CallControlApplication.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/call_control_applications/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.CallControlApplication)

    def test_is_creatable(self, request_mock):
        resource = telnyx.CallControlApplication.create(
            connection_name="foo",
            webhook_event_url="http://foo.com",
            application_name="foo",
        )
        request_mock.assert_requested("post", "/v2/call_control_applications")
        assert isinstance(resource, telnyx.CallControlApplication)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_saveable(self, request_mock):
        call_control_application = telnyx.CallControlApplication.retrieve(
            TEST_RESOURCE_ID
        )
        call_control_application.connection_name = "foo"
        call_control_application.webhook_event_url = "http://foo.com"
        call_control_application.application_name = "foo"
        resource = call_control_application.save()
        request_mock.assert_requested(
            "patch", "/v2/call_control_applications/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.CallControlApplication)
        assert resource is call_control_application

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_modifiable(self, request_mock):
        resource = telnyx.CallControlApplication.modify(
            TEST_RESOURCE_ID,
            connection_name="foo",
            webhook_event_url="http://foo.com",
            application_name="foo",
        )
        request_mock.assert_requested(
            "patch", "/v2/call_control_applications/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.CallControlApplication)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_deletable(self, request_mock):
        resource = telnyx.CallControlApplication.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/call_control_applications/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "6a09cdc3-8948-47f0-aa62-74ac943d6c58"


class TestDocument(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.Document.list()
        request_mock.assert_requested("get", "/v2/documents")
        assert isinstance(resources.data, list)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_retrievable(self, request_mock):
        telnyx.Document.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/documents/%s" % TEST_RESOURCE_ID)

    def test_is_creatable(self, request_mock):
        resource = telnyx.Document.create(
            filename="test-document.pdf",
            url="https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf",
            file="https://test.com",
        )
        resource
        request_mock.assert_requested("post", "/v2/documents")

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_modifiable(self, request_mock):
        resource = telnyx.Document.modify(
            TEST_RESOURCE_ID,
            active=False,
            webhook_event_url="https://update.com",
            application_name="updated name",
        )
        resource
        request_mock.assert_requested("patch", "/v2/documents/%s" % TEST_RESOURCE_ID)

    @pytest.mark.skip(reason="Nested get")
    def test_is_deletable(self, request_mock):
        resource = telnyx.Document.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested("delete", "/v2/documents/%s" % TEST_RESOURCE_ID)

    def test_document_links(self, request_mock):
        resources = telnyx.DocumentLink.list()
        request_mock.assert_requested("get", "/v2/document_links")
        assert isinstance(resources.data, list)

    @pytest.mark.skip(reason="Nested get")
    def test_download(self, request_mock):
        resource = telnyx.Document.retrieve(TEST_RESOURCE_ID)
        resource.download()



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    UpdateableAPIResource,
)


class BulkCredential(
    CreateableAPIResource, DeletableAPIResource, UpdateableAPIResource
):
    OBJECT_NAME = "bulk_credential"



from __future__ import absolute_import, division, print_function

import telnyx.request_metrics


class TestRequestMetrics(object):
    def test_payload(self):
        metrics = telnyx.request_metrics.RequestMetrics("1", 1000)

        assert metrics.payload() == {"request_id": "1", "request_duration_ms": 1000}



from __future__ import absolute_import, division, print_function

from six.moves.urllib.parse import quote_plus

from telnyx import api_requestor, util
from telnyx.api_resources.abstract.api_resource import APIResource


class UpdateableAPIResource(APIResource):
    @classmethod
    def _modify(cls, url, api_key=None, **params):
        requestor = api_requestor.APIRequestor(api_key)
        params = util.rewrite_reserved_words(params)
        response, api_key = requestor.request("patch", url, params)
        return util.convert_to_telnyx_object(response, api_key)

    @classmethod
    def modify(cls, sid, **params):
        url = "%s/%s" % (cls.class_url(), quote_plus(util.utf8(sid)))
        return cls._modify(url, **params)

    @classmethod
    def _putify(cls, url, api_key=None, **params):
        requestor = api_requestor.APIRequestor(api_key)
        params = util.rewrite_reserved_words(params)
        response, api_key = requestor.request("put", url, params)
        return util.convert_to_telnyx_object(response, api_key)

    @classmethod
    def putify(cls, sid, **params):
        url = "%s/%s" % (cls.class_url(), quote_plus(util.utf8(sid)))
        return cls._putify(url, **params)

    def save(self):
        updated_params = self.serialize(None)

        if updated_params:
            self.refresh_from(
                self.request(self.save_method(), self.instance_url(), updated_params)
            )
        else:
            util.logger.debug("Trying to save already saved object %r", self)
        return self

    def save_method(self):
        if self.id is None:
            return "post"
        else:
            return "patch"



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "3fa85f64-5717-4562-b3fc-2c963f66afa6"


class TestMessagingProfile(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.MessagingProfile.list()
        request_mock.assert_requested("get", "/v2/messaging_profiles")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.MessagingProfile)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_retrievable(self, request_mock):
        resource = telnyx.MessagingProfile.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/messaging_profiles/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.MessagingProfile)

    @pytest.mark.skip(reason="Prism Mock bug in spec file (request)")
    def test_is_creatable(self, request_mock):
        resource = telnyx.MessagingProfile.create(name="my-profile")
        request_mock.assert_requested("post", "/v2/messaging_profiles")
        assert isinstance(resource, telnyx.MessagingProfile)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_saveable(self, request_mock):
        messaging_profile = telnyx.MessagingProfile.retrieve(TEST_RESOURCE_ID)
        messaging_profile.name = "value"
        resource = messaging_profile.save()
        request_mock.assert_requested(
            "patch", "/v2/messaging_profiles/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.MessagingProfile)
        assert resource is messaging_profile

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_modifiable(self, request_mock):
        resource = telnyx.MessagingProfile.modify(TEST_RESOURCE_ID, name="Test")
        request_mock.assert_requested(
            "patch", "/v2/messaging_profiles/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.MessagingProfile)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_deletable(self, request_mock):
        resource = telnyx.MessagingProfile.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/messaging_profiles/%s" % TEST_RESOURCE_ID
        )

    @pytest.mark.skip(reason="Might be deprecated")
    def test_can_call_messaging_phone_numbers(self, request_mock):
        resources = telnyx.MessagingProfile.list_phone_numbers(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/messaging_profiles/%s/phone_numbers" % TEST_RESOURCE_ID
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.MessagingPhoneNumber)

    @pytest.mark.skip(reason="Might be deprecated")
    def test_can_call_phone_numbers(self, request_mock):
        messaging_profile = telnyx.MessagingProfile.retrieve(TEST_RESOURCE_ID)
        resources = messaging_profile.phone_numbers()
        request_mock.assert_requested(
            "get", "/v2/messaging_profiles/%s/phone_numbers" % TEST_RESOURCE_ID
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.MessagingPhoneNumber)

    @pytest.mark.skip(reason="Might be deprecated")
    def test_can_call_phone_numbers_forward_params(self, request_mock):
        messaging_profile = telnyx.MessagingProfile.retrieve(TEST_RESOURCE_ID)
        resources = messaging_profile.phone_numbers(page={"size": 10})
        request_mock.assert_requested(
            "get",
            "/v2/messaging_profiles/%s/phone_numbers" % TEST_RESOURCE_ID,
            {"page": {"size": 10}},
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.MessagingPhoneNumber)

    def test_can_call_messaging_short_codes(self, request_mock):
        resources = telnyx.MessagingProfile.list_short_codes(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/messaging_profiles/%s/short_codes" % TEST_RESOURCE_ID
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.ShortCode)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_can_call_short_codes(self, request_mock):
        messaging_profile = telnyx.MessagingProfile.retrieve(TEST_RESOURCE_ID)
        resources = messaging_profile.short_codes()
        request_mock.assert_requested(
            "get", "/v2/messaging_profiles/%s/short_codes" % TEST_RESOURCE_ID
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.ShortCode)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_can_call_short_codes_forward_params(self, request_mock):
        messaging_profile = telnyx.MessagingProfile.retrieve(TEST_RESOURCE_ID)
        resources = messaging_profile.short_codes(page={"size": 20})
        request_mock.assert_requested(
            "get",
            "/v2/messaging_profiles/%s/short_codes" % TEST_RESOURCE_ID,
            {"page": {"size": 20}},
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.ShortCode)

    @pytest.mark.skip(reason="Might be deprecated")
    def test_can_call_metrics(self, request_mock):
        messaging_profile = telnyx.MessagingProfile.retrieve(TEST_RESOURCE_ID)
        resources = messaging_profile.metrics()
        request_mock.assert_requested(
            "get", "/v2/messaging_profiles/%s/metrics" % TEST_RESOURCE_ID
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.MessagingProfile)



from __future__ import absolute_import, division, print_function

import pytest

import telnyx


class TestUpdateableAPIResource(object):
    class MyUpdateable(telnyx.api_resources.abstract.UpdateableAPIResource):
        OBJECT_NAME = "myupdateable"

    @pytest.fixture
    def obj(self, request_mock):
        request_mock.stub_request(
            "patch",
            "/v2/myupdateables/myid",
            {"id": "myid", "thats": "it", "from": "outside"},
            rheaders={"request-id": "req_id"},
        )

        return self.MyUpdateable.construct_from(
            {
                "id": "myid",
                "foo": "bar",
                "baz": "boz",
                "from": "inside",
                "nested_object": {"size": "l", "score": 4, "height": 10},
            },
            "mykey",
        )

    def checkSave(self, obj):
        assert obj is obj.save()

        assert obj.thats == "it"
        # TODO: Should we force id to be retained?
        # assert obj.id == 'myid'
        with pytest.raises(AttributeError):
            obj.baz

    def test_save(self, request_mock, obj):
        obj.baz = "updated"
        obj.other = "newval"
        obj.nested_object.size = "m"
        obj.nested_object.info = "a2"
        obj.nested_object.height = None

        self.checkSave(obj)

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {
                "baz": "updated",
                "other": "newval",
                "nested_object": {"size": "m", "info": "a2", "height": ""},
            },
            None,
        )

        assert obj.last_response is not None
        assert obj.last_response.request_id == "req_id"

        # Saving again should not cause any request.
        request_mock.reset_mock()
        self.checkSave(obj)
        request_mock.assert_no_request()

        # Setting the same value should cause a request.
        request_mock.stub_request(
            "patch", "/v2/myupdateables/myid", {"id": "myid", "thats": "it"}
        )

        obj.thats = "it"
        self.checkSave(obj)

        request_mock.assert_requested(
            "patch", "/v2/myupdateables/myid", {"thats": "it"}, None
        )

        # Changing the value should cause a request.
        request_mock.stub_request(
            "patch", "/v2/myupdateables/myid", {"id": "myid", "thats": "it"}
        )

        obj.id = "myid"
        obj.thats = "updated"
        self.checkSave(obj)

        request_mock.assert_requested(
            "patch", "/v2/myupdateables/myid", {"thats": "updated"}, None
        )

    def test_add_key_to_nested_object(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {"size": "l", "score": 4, "height": 10}},
            "mykey",
        )

        acct.legal_entity["first_name"] = "bob"

        assert acct is acct.save()

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {"legal_entity": {"first_name": "bob"}},
            None,
        )

    def test_save_nothing(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "nested_object": {"key": "value"}}, "mykey"
        )

        assert acct is acct.save()

        request_mock.assert_no_request()

    def test_do_not_replace_nested_object(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {"last_name": "smith"}}, "mykey"
        )

        acct.legal_entity = {"first_name": "bob"}

        assert acct is acct.save()

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {"legal_entity": {"first_name": "bob", "last_name": "smith"}},
            None,
        )

    def test_array_setting(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {}}, "mykey"
        )

        acct.legal_entity.additional_owners = [{"first_name": "Bob"}]

        assert acct is acct.save()

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {"legal_entity": {"additional_owners": [{"first_name": "Bob"}]}},
            None,
        )

    def test_array_none(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {"additional_owners": None}}, "mykey"
        )

        acct.foo = "bar"

        assert acct is acct.save()

        request_mock.assert_requested(
            "patch", "/v2/myupdateables/myid", {"foo": "bar"}, None
        )

    def test_array_insertion(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {"additional_owners": []}}, "mykey"
        )

        acct.legal_entity.additional_owners.append({"first_name": "Bob"})

        assert acct is acct.save()

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {"legal_entity": {"additional_owners": {"0": {"first_name": "Bob"}}}},
            None,
        )

    def test_array_update(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {
                "id": "myid",
                "legal_entity": {
                    "additional_owners": [{"first_name": "Bob"}, {"first_name": "Jane"}]
                },
            },
            "mykey",
        )

        acct.legal_entity.additional_owners[1].first_name = "Janet"

        assert acct is acct.save()

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {
                "legal_entity": {
                    "additional_owners": {"0": {}, "1": {"first_name": "Janet"}}
                }
            },
            None,
        )

    def test_array_noop(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {
                "id": "myid",
                "legal_entity": {"additional_owners": [{"first_name": "Bob"}]},
                "currencies_supported": ["usd", "cad"],
            },
            "mykey",
        )

        assert acct is acct.save()

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {"legal_entity": {"additional_owners": {"0": {}}}},
            None,
        )

    def test_hash_noop(self, request_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {"address": {"line1": "1 Two Three"}}},
            "mykey",
        )

        assert acct is acct.save()

        request_mock.assert_no_request()

    def test_save_replace_nested_object_with_number(self, request_mock, obj):
        obj.baz = "updated"
        obj.other = "newval"
        obj.nested_object = 3

        self.checkSave(obj)

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {"baz": "updated", "other": "newval", "nested_object": 3},
            None,
        )

    def test_save_do_not_overwrite_nested_object(self, request_mock, obj):
        old_nested_object = obj._previous.get("nested_object")
        obj.nested_object = {}
        self.checkSave(obj)

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {"nested_object": old_nested_object},
            None,
        )

    def test_save_do_not_replace_nested_object(self, request_mock, obj):
        old_nested_object = obj._previous.get("nested_object")
        obj.baz = "updated"
        obj.other = "newval"
        obj.nested_object = {"size": "m", "info": "a2", "score": 4}

        self.checkSave(obj)

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {
                "baz": "updated",
                "other": "newval",
                "nested_object": {
                    "size": "m",
                    "info": "a2",
                    "height": old_nested_object.get("height"),
                    "score": 4,
                },
            },
            None,
        )

    def test_save_update_nested_object(self, request_mock, obj):
        obj.baz = "updated"
        obj.other = "newval"
        obj.nested_object.update({"size": "m", "info": "a2", "score": 4})

        self.checkSave(obj)

        request_mock.assert_requested(
            "patch",
            "/v2/myupdateables/myid",
            {
                "baz": "updated",
                "other": "newval",
                "nested_object": {"size": "m", "info": "a2", "score": 4},
            },
            None,
        )

    def test_save_method(self):
        v = self.MyUpdateable.construct_from({"id": None, "foo": "bar"}, "mykey")

        assert self.MyUpdateable.save_method(v) == "post"

        v = self.MyUpdateable.construct_from({"id": "123", "foo": "bar"}, "mykey")

        assert self.MyUpdateable.save_method(v) == "patch"

    def test_save_reserved_word(self, request_mock, obj):
        obj.from_ = "outside"

        self.checkSave(obj)

        request_mock.assert_requested(
            "patch", "/v2/myupdateables/myid", {"from": "outside"}, None
        )

        assert obj.from_ == "outside"



from __future__ import absolute_import, division, print_function

import telnyx

CONFERENCE_ID = "3fa85f64-5717-4562-b3fc-2c963f66afa6"
_call_control_id = "AgDIxmoRX6QMuaIj_uXRXnPAXP0QlNfXczRrZvZakpWxBlpw48KyZQ=="


def create_conference():
    return telnyx.Conference.create(
        name="Business",
        call_control_id=_call_control_id,
        client_state="aGF2ZSBhIG5pY2UgZGF5ID1d",
        command_id="891510ac-f3e4-11e8-af5b-de00688a4901",
    )


class TestConference(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.Conference.list()
        request_mock.assert_requested("get", "/v2/conferences")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.Conference)

    def test_is_retrievable(self, request_mock):
        resources = telnyx.Conference.retrieve(CONFERENCE_ID)
        request_mock.assert_requested("get", "/v2/conferences/%s" % CONFERENCE_ID)
        assert isinstance(resources, telnyx.Conference)

    def test_is_creatable(self, request_mock):
        resource = create_conference()
        request_mock.assert_requested("post", "/v2/conferences")
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_join(self, request_mock):
        resource = create_conference()
        resource.join(call_control_id=_call_control_id)
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/join" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_join(self, request_mock):
        resource = create_conference()
        resource.create_join(CONFERENCE_ID, call_control_id=_call_control_id)
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/join" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_mute(self, request_mock):
        resource = create_conference()
        resource.mute(call_control_ids=[_call_control_id])
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/mute" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_mute(self, request_mock):
        resource = create_conference()
        resource.create_mute(CONFERENCE_ID, call_control_ids=[_call_control_id])
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/mute" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_unmute(self, request_mock):
        resource = create_conference()
        resource.unmute(call_control_ids=[_call_control_id])
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/unmute" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_unmute(self, request_mock):
        resource = create_conference()
        resource.create_unmute(CONFERENCE_ID, call_control_ids=[_call_control_id])
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/unmute" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_hold(self, request_mock):
        resource = create_conference()
        resource.hold(
            call_control_ids=[_call_control_id],
            audio_url="http://example.com/message.wav",
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/hold" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_hold(self, request_mock):
        resource = create_conference()
        resource.create_hold(
            CONFERENCE_ID,
            call_control_ids=[_call_control_id],
            audio_url="http://example.com/message.wav",
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/hold" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_unhold(self, request_mock):
        resource = create_conference()
        resource.unhold(
            call_control_ids=[_call_control_id],
            audio_url="http://example.com/message.wav",
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/unhold" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_unhold(self, request_mock):
        resource = create_conference()
        resource.create_unhold(
            CONFERENCE_ID,
            call_control_ids=[_call_control_id],
            audio_url="http://example.com/message.wav",
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/unhold" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_speak(self, request_mock):
        resource = create_conference()
        resource.speak(
            call_control_ids=[_call_control_id],
            voice="female",
            payload="I hope this unit tests passes",
            language="en-AU",
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/speak" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_speak(self, request_mock):
        resource = create_conference()
        resource.create_speak(
            CONFERENCE_ID,
            voice="female",
            payload="I hope this unit tests passes",
            language="en-AU",
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/speak" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_play(self, request_mock):
        resource = create_conference()
        resource.call_control_id = CONFERENCE_ID
        resource.play(audio_url="http://www.example.com/sounds/greeting.wav")
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/play" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_play(self, request_mock):
        resource = create_conference()
        resource.create_play(
            CONFERENCE_ID, audio_url="http://www.example.com/sounds/greeting.wav"
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/play" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_stop(self, request_mock):
        resource = create_conference()
        resource.call_control_id = CONFERENCE_ID
        resource.stop()
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/stop" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_stop(self, request_mock):
        resource = create_conference()
        resource.create_stop(CONFERENCE_ID)
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/stop" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_record_start(self, request_mock):
        resource = create_conference()
        resource.record_start(
            call_control_ids=[_call_control_id], format="wav", channels="single"
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/record_start" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_record_start(self, request_mock):
        resource = create_conference()
        resource.create_record_start(CONFERENCE_ID, format="wav", channels="single")
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/record_start" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_record_stop(self, request_mock):
        resource = create_conference()
        resource.record_stop(call_control_ids=[_call_control_id])
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/record_stop" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_create_record_stop(self, request_mock):
        resource = create_conference()
        resource.create_record_stop(CONFERENCE_ID)
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/record_stop" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_update(self, request_mock):
        resource = create_conference()
        resource.create_update(
            CONFERENCE_ID,
            call_control_id=_call_control_id,
            supervisor_role="whisper",
            command_id="891510ac-f3e4-11e8-af5b-de00688a4901",
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/update" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)

    def test_can_call_leave(self, request_mock):
        resource = create_conference()
        resource.create_leave(
            CONFERENCE_ID,
            call_control_id=_call_control_id,
            supervisor_role="whisper",
            command_id="891510ac-f3e4-11e8-af5b-de00688a4901",
        )
        request_mock.assert_requested(
            "post", "/v2/conferences/%s/actions/leave" % CONFERENCE_ID
        )
        assert isinstance(resource, telnyx.Conference)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import CreateableAPIResource, ListableAPIResource


class SIMCardOrder(CreateableAPIResource, ListableAPIResource):
    OBJECT_NAME = "sim_card_order"


class SIMCardOrderPreview(CreateableAPIResource):
    OBJECT_NAME = "sim_card_order_preview"

    @classmethod
    def class_url(cls):
        return "/v2/sim_card_order_preview"



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_FAX_CONNECTION_ID = "0239852093"
TEST_SRC_LONG_CODE = "+13125550100"
TEST_DST = "+17735550100"
TEST_MEDIA_URL = "http://www.africau.edu/images/default/sample.pdf"
TEST_RESOURCE_ID = "3fa85f64-5717-4562-b3fc-2c963f66afa6"


class TestFax(object):
    def test_can_create_standard_fax(self, request_mock):
        resource = telnyx.Fax.create(
            from_=TEST_SRC_LONG_CODE,
            to=TEST_DST,
            media_url=TEST_MEDIA_URL,
            connection_id=TEST_FAX_CONNECTION_ID,
        )
        request_mock.assert_requested("post", "/v2/faxes")
        print(resource)
        print(type(resource))
        assert isinstance(resource, telnyx.Fax)

    @pytest.mark.skip(reason="Fax needs correct input")
    def test_can_fax_refresh(self, request_mock):
        resource = telnyx.Fax.retrieve(TEST_RESOURCE_ID)
        resource.create_refresh(id=TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "post", "/v2/faxes/%s/actions/refresh" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


@pytest.mark.skip(reason="Not implemented yet")
class TestCampaign(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.Campaign.list()
        request_mock.assert_requested("get", "/10dlc/campaign")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.Campaign)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.Campaign.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/campaign/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.Campaign)

    def test_is_creatable(self, request_mock):
        resource = telnyx.FaxApplication.create(
            active=True,
            application_name="Test Name",
            webhook_event_url="https://test.com",
        )
        request_mock.assert_requested("post", "/v2/fax_applications")
        assert isinstance(resource, telnyx.FaxApplication)

    def test_is_deletable(self, request_mock):
        resource = telnyx.Campaign.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested("delete", "/10dlc/campaign/%s" % TEST_RESOURCE_ID)



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


class TestMedia(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.Media.list()
        request_mock.assert_requested("get", "/v2/media")
        assert isinstance(resources.data, list)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.Media.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/media/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.Media)

    def test_is_creatable(self, request_mock):
        telnyx.Media.create(media_url="http://www.example.com/audio.mp3")
        request_mock.assert_requested("post", "/v2/media")

    @pytest.mark.skip(reason="Request body /w object")
    def test_is_modifiable(self, request_mock):
        resource = telnyx.Media.putify(TEST_RESOURCE_ID)
        request_mock.assert_requested("put", "/v2/media/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.Media)

    @pytest.mark.skip(reason="Invalid Url")
    def test_is_deletable(self, request_mock):
        resource = telnyx.Media.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested("delete", "/v2/media/%s" % TEST_RESOURCE_ID)

    @pytest.mark.skip(reason="Nested download")
    def test_can_download(self, request_mock):
        resource = telnyx.Media.retrieve(TEST_RESOURCE_ID)
        resource.download(id=TEST_RESOURCE_ID)



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


class TestManagedAccount(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.ManagedAccount.list()
        request_mock.assert_requested("get", "/v2/managed_accounts")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.ManagedAccount)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.ManagedAccount.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/managed_accounts/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.ManagedAccount)

    def test_is_creatable(self, request_mock):
        telnyx.ManagedAccount.create(business_name="Larry's Cat Food Inc")
        request_mock.assert_requested("post", "/v2/managed_accounts")

    def test_is_modifiable(self, request_mock):
        resource = telnyx.ManagedAccount.modify(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "patch", "/v2/managed_accounts/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.ManagedAccount)

    @pytest.mark.skip(reason="Empty body")
    def test_can_enable(self, request_mock):
        resource = telnyx.ManagedAccount.retrieve(TEST_RESOURCE_ID)
        resource.enable()
        request_mock.assert_requested(
            "post", "/v2/managed_accounts/%s/actions/enable" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.ManagedAccount)

    @pytest.mark.skip(reason="Empty body")
    def test_can_disable(self, request_mock):
        resource = telnyx.ManagedAccount.retrieve(TEST_RESOURCE_ID)
        resource.disable()
        request_mock.assert_requested(
            "post", "/v2/managed_accounts/%s/actions/disable" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.ManagedAccount)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "check_registration_status",
    path="actions/check_registration_status",
    operations=["create"],
)
class CredentialConnection(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "credential_connection"

    def check_registration_status(self, **params):
        return self.create_check_registration_status(**params)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract.api_resource import APIResource


class DeletableAPIResource(APIResource):
    def delete(self, **params):
        self.refresh_from(self.request("delete", self.instance_url(), params))
        return self



from __future__ import absolute_import, division, print_function

import telnyx

TEST_RESOURCE_ID = "6a09cdc3-8948-47f0-aa62-74ac943d6c58"


class TestConnection(object):
    connection_types = (
        telnyx.IPConnection,
        telnyx.CredentialConnection,
        telnyx.IP,
        telnyx.FQDNConnection,
        telnyx.FQDN,
    )

    def test_is_listable(self, request_mock):
        resources = telnyx.Connection.list()
        request_mock.assert_requested("get", "/v2/connections")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], self.connection_types)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.Connection.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/connections/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, self.connection_types)



from __future__ import absolute_import, division, print_function

import telnyx

TEST_RESOURCE_ID = "6a09cdc3-8948-47f0-aa62-74ac943d6c58"


class TestWirelessDetailRecords(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.WirelessDetailRecordsReports.list()
        request_mock.assert_requested("get", "/v2/wireless/detail_records_reports")
        assert isinstance(resources.data, list)

    def test_is_retrievable(self, request_mock):
        telnyx.WirelessDetailRecordsReports.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/wireless/detail_records_reports/%s" % TEST_RESOURCE_ID
        )

    def test_is_creatable(self, request_mock):
        telnyx.WirelessDetailRecordsReports.create()
        request_mock.assert_requested("post", "/v2/wireless/detail_records_reports")

    def test_is_deletable(self, request_mock):
        resource = telnyx.WirelessDetailRecordsReports.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/wireless/detail_records_reports/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods("generate", path="generate", operations=["create"])
@nested_resource_class_methods(
    "generate_stream", path="generate_stream", operations=["create"]
)
@nested_resource_class_methods("summarize", path="summarize", operations=["create"])
@nested_resource_class_methods("embeddings", path="embeddings", operations=["create"])
@nested_resource_class_methods(
    "similarity_search", path="embeddings/similarity-search", operations=["create"]
)
class AI(CreateableAPIResource, DeletableAPIResource, ListableAPIResource):
    OBJECT_NAME = "ai"

    def generate(self, **params):
        return AI.create_generate(**params)

    def generate_stream(self, **params):
        return AI.create_generate_stream(**params)

    def summarize(self, **params):
        return AI.create_summarize(**params)

    def embeddings(self, **params):
        return AI.create_embeddings(**params)

    def similarity_search(self, **params):
        return AI.create_simalarity_search



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import SingletonAPIResource


class PublicKey(SingletonAPIResource):
    OBJECT_NAME = "public_key"



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0"


@pytest.mark.skip(reason="Prism Mock returns unexpected non-200 response")
class TestAddress(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.AccessControlIP.list()
        request_mock.assert_requested("get", "/v2/access_control_ips")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.AccessControlIP)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.AccessControlIP.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/access_control_ips/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.AccessControlIP)

    def test_is_creatable(self, request_mock):
        resource = telnyx.AccessControlIP.create(ip_address="100.101.102.103")
        request_mock.assert_requested("post", "/v2/access_control_ips")
        assert isinstance(resource, telnyx.AccessControlIP)

    def test_is_deletable(self, request_mock):
        resource = telnyx.AccessControlIP.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/access_control_ips/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

import telnyx


class TestInboundChannels(object):
    def test_is_retrievable(self, request_mock):
        resource = telnyx.InboundChannel.retrieve()
        request_mock.assert_requested("get", "/v2/phone_numbers/inbound_channels")
        assert isinstance(resource, telnyx.InboundChannel)

    def test_is_saveable(self, request_mock):
        inbound_channels = telnyx.InboundChannel.retrieve()
        inbound_channels.channels = 10
        resource = inbound_channels.save()
        request_mock.assert_requested("patch", "/v2/phone_numbers/inbound_channels")
        assert isinstance(resource, telnyx.InboundChannel)
        assert resource is inbound_channels

    def test_is_modifiable(self, request_mock):
        resource = telnyx.InboundChannel.modify(channels=10)
        request_mock.assert_requested("patch", "/v2/phone_numbers/inbound_channels")
        assert isinstance(resource, telnyx.InboundChannel)



from __future__ import absolute_import, division, print_function

import telnyx

TEST_RESOURCE_ID = "6a09cdc3-8948-47f0-aa62-74ac943d6c58"


class TestSIMCardOrder(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.SIMCardOrder.list()
        request_mock.assert_requested("get", "/v2/sim_card_orders")
        assert isinstance(resources.data, list)

    def test_is_retrievable(self, request_mock):
        telnyx.SIMCardOrder.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/sim_card_orders/%s" % TEST_RESOURCE_ID
        )

    def test_is_creatable(self, request_mock):
        resource = telnyx.SIMCardOrder.create(address_id=TEST_RESOURCE_ID, quantity=23)
        request_mock.assert_requested("post", "/v2/sim_card_orders")
        resource

    def test_can_sim_preview(self, request_mock):
        telnyx.SIMCardOrderPreview.create(address_id=TEST_RESOURCE_ID, quantity=23)



from __future__ import absolute_import, division, print_function

import telnyx


class TestPortabilityCheck(object):
    def test_is_creatable(self, request_mock):
        telnyx.PortabilityCheck.create()
        request_mock.assert_requested("post", "/v2/portability_checks")



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "enable_emergency", path="actions/enable_emergency", operations=["create"]
)
class PhoneNumberConfiguration(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "phone_number_configuration"

    def enable_emergency(self, **params):
        return self.create_enable_emergency(**params)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
)


class DynamicEmergencyAddress(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "dynamic_emergency_address"

    @classmethod
    def class_url(cls):
        return "/v2/dynamic_emergency_addresses"



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import ListableAPIResource


class MessagingUrlDomain(ListableAPIResource):
    OBJECT_NAME = "messaging_url_domain"



from __future__ import absolute_import, division, print_function

import six
from six.moves.urllib.parse import quote_plus

from telnyx import error, util
from telnyx.telnyx_object import TelnyxObject


class APIResource(TelnyxObject):
    @classmethod
    def retrieve(cls, id, api_key=None, **params):
        instance = cls(id, api_key, **params)
        response = instance.request("get", instance.instance_url())

        # It's not always the case that `cls.retrieve()` should return an
        # object of type `cls`, as the response's `record_type` might indicate
        # another `telnyx.api_resources.*` type.
        #
        # If this is the case, we simply return the already initialised
        # `response` instance.
        #
        # Otherwise, we will have an object of type `TelnyxObject`, the
        # default type returned by `util.convert_to_telnyx_object`, and we
        # promote the object back to an `APIResource`.
        if isinstance(response, TelnyxObject) and not isinstance(response, APIResource):
            instance.refresh_from(response)
            return instance
        else:
            return response

    def refresh(self):
        self.refresh_from(self.request("get", self.instance_url()))
        return self

    @classmethod
    def class_url(cls):
        if cls == APIResource:
            raise NotImplementedError(
                "APIResource is an abstract class.  You should perform "
                "actions on its subclasses (e.g. MessagingProfile)"
            )
        # Namespaces are separated in object names with periods (.) and in URLs
        # with forward slashes (/), so replace the former with the latter.
        base = cls.OBJECT_NAME.replace(".", "/")
        return "/v2/%ss" % (base,)

    def instance_url(self):
        id = self.get("id")

        if not isinstance(id, six.string_types):
            raise error.InvalidRequestError(
                "Could not determine which URL to request: %s instance "
                "has invalid ID: %r, %s. ID should be of type `str` (or"
                " `unicode`)" % (type(self).__name__, id, type(id)),
                "id",
            )

        id = util.utf8(id)
        base = self.class_url()
        extn = quote_plus(id)
        return "%s/%s" % (base, extn)



import pytest

import telnyx

TEST_RESOURCE_ID = "3fa85f64-5717-4562-b3fc-2c963f66afa6"


class TestRoom(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.Room.list()
        request_mock.assert_requested("get", "/v2/rooms")
        assert isinstance(resources.data, list)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.Room.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/rooms/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.Room)

    def test_is_creatable(self, request_mock):
        telnyx.Room.create()
        request_mock.assert_requested("post", "/v2/rooms")

    def test_is_modifiable(self, request_mock):
        telnyx.Room.modify(
            TEST_RESOURCE_ID, enable_recording=False, max_participants=20
        )
        request_mock.assert_requested("patch", "/v2/rooms/%s" % TEST_RESOURCE_ID)

    @pytest.mark.skip(reason="204 Response (silent)")
    def test_is_deletable(self, request_mock):
        resource = telnyx.Room.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested("delete", "/v2/rooms/%s" % TEST_RESOURCE_ID)

    @pytest.mark.skip(reason="Nest List")
    def test_can_list_sessions(self, request_mock):
        resource = telnyx.Room.retrieve(TEST_RESOURCE_ID)
        resource.list_sessions()

    def test_can_generate_token(self, request_mock):
        resource = telnyx.Room.retrieve(TEST_RESOURCE_ID)
        resource.create_generate_token(id=TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "post", "/v2/rooms/%s/actions/generate_join_client_token" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.Room)

    @pytest.mark.skip(reason="Needs token")
    def test_can_refresh_token(self, request_mock):
        resource = telnyx.Room.retrieve(TEST_RESOURCE_ID)
        resource.create_refresh_token(id="TEST_RESOURCE_ID")
        request_mock.assert_requested(
            "post", "/v2/rooms/%s/actions/refresh_client_token" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.Room)



import pytest
import telnyx

TEST_RESOURCE_ID = "12ade33a-21c0-473b-b055-b3c836e1c292"
TEST_PHONE_NUMBER = "+13125000000"
VERIFY_CODE = "12345"
VERIFY_PROFILE = "12ade33a-21c0-473b-b055-b3c836e1c292"


class TestVerification(object):
    def test_is_retrievable(self, request_mock):
        resource = telnyx.Verification.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/verifications/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.Verification)

    def test_can_verify_by_phone_number(self, request_mock):
        telnyx.Verification.verify_by_phone_number(
            phone_number=TEST_PHONE_NUMBER,
            code=VERIFY_CODE,
            verify_profile_id=VERIFY_PROFILE,
        )
        request_mock.assert_requested(
            "post",
            "/v2/verifications/by_phone_number/%s/actions/verify" % TEST_PHONE_NUMBER,
        )

    def test_can_verify_by_id(self, request_mock):
        telnyx.Verification.verify_by_id(
            verification_id=TEST_RESOURCE_ID,
            code=VERIFY_CODE,
            verify_profile_id=VERIFY_PROFILE,
        )
        request_mock.assert_requested(
            "post",
            "/v2/verifications/%s/actions/verify" % TEST_RESOURCE_ID,
        )

    def test_can_list_verifications(self, request_mock):
        resources = telnyx.Verification.by_phone_number(TEST_PHONE_NUMBER)
        request_mock.assert_requested("get", "/v2/verifications/by_phone_number/%s" %TEST_PHONE_NUMBER)
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.Verification)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_verify_by_sms(self, request_mock):
        telnyx.Verification.sms(
            phone_number=TEST_PHONE_NUMBER, verify_profile_id=VERIFY_PROFILE
        )
        request_mock.assert_requested("post", "/v2/verifications/sms")

    def test_verify_by_call(self, request_mock):
        telnyx.Verification.call(
            phone_number=TEST_PHONE_NUMBER, verify_profile_id=VERIFY_PROFILE
        )
        request_mock.assert_requested("post", "/v2/verifications/call")

    def test_verify_by_flashcall(self, request_mock):
        telnyx.Verification.flashcall(
            phone_number=TEST_PHONE_NUMBER, verify_profile_id=VERIFY_PROFILE
        )
        request_mock.assert_requested("post", "/v2/verifications/flashcall")



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "list_external_vetting", path="externalVetting", operations=["list"]
)
@nested_resource_class_methods(
    "order_external_vetting", path="externalVetting", operations=["create"]
)
class Brand(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "brand"

    @classmethod
    def class_url(cls):
        return "/10dlc/brand"

    def list_external_vetting(self, **params):
        return Brand.list_external_vetting(self.name, **params)

    def operation_status(self, **params):
        return Brand.create_external_vetting(self.id, **params)



from __future__ import absolute_import, division, print_function

import datetime
import json
import pickle
from copy import copy, deepcopy

import pytest
import six

import telnyx

SAMPLE_FOO = json.loads(
    """
{
  "record_type": "foo",
  "id": "3fa85f64-5717-4562-b3fc-2c963f66afa6",
  "organization_id": "3fa85f64-5717-4562-b3fc-2c963f66afa6",
  "name": "New Messaging Profile",
  "created_at": "2019-01-23T18:10:02.574Z",
  "updated_at": "2019-01-23T18:10:02.574Z",
  "calls": {
    "outbound": [],
    "inbound": [
        {
            "record_type": "call",
            "started_at": "2019-01-23T18:10:02.574Z",
            "ended_at": "2019-01-23T18:10:02.574Z",
            "metadata": {
                "foo": "bar"
            }
        }
    ],
    "on_net": []
    }
}
"""
)


class TestTelnyxObject(object):
    def test_initializes_with_parameters(self):
        obj = telnyx.telnyx_object.TelnyxObject(
            "foo", "bar", myparam=5, yourparam="boo"
        )

        assert obj.id == "foo"
        assert obj.api_key == "bar"

    def test_access(self):
        obj = telnyx.telnyx_object.TelnyxObject("myid", "mykey", myparam=5)

        # Empty
        with pytest.raises(AttributeError):
            obj.myattr
        with pytest.raises(KeyError):
            obj["myattr"]
        assert obj.get("myattr", "def") == "def"
        assert obj.get("myattr") is None

        # Setters
        obj.myattr = "myval"
        obj["myitem"] = "itval"
        assert obj.setdefault("mydef", "sdef") == "sdef"

        # Getters
        assert obj.setdefault("myattr", "sdef") == "myval"
        assert obj.myattr == "myval"
        assert obj["myattr"] == "myval"
        assert obj.get("myattr") == "myval"

        assert sorted(obj.keys()) == ["id", "myattr", "mydef", "myitem"]

        assert sorted(obj.values()) == ["itval", "myid", "myval", "sdef"]

        # Illegal operations
        with pytest.raises(ValueError):
            obj.foo = ""

    def test_refresh_from(self, mocker):
        obj = telnyx.telnyx_object.TelnyxObject.construct_from(
            {"foo": "bar", "trans": "me"}, "mykey"
        )

        assert obj.api_key == "mykey"
        assert obj.foo == "bar"
        assert obj["trans"] == "me"
        assert obj.last_response is None

        last_response = mocker.Mock()
        obj.refresh_from(
            {"foo": "baz", "johnny": 5}, "key2", last_response=last_response
        )

        assert obj.johnny == 5
        assert obj.foo == "baz"
        with pytest.raises(AttributeError):
            obj.trans
        assert obj.api_key == "key2"
        assert obj.last_response == last_response

        obj.refresh_from({"trans": 4, "metadata": {"amount": 42}}, "key2", True)

        assert obj.foo == "baz"
        assert obj.trans == 4

    def test_passing_nested_refresh(self):
        obj = telnyx.telnyx_object.TelnyxObject.construct_from(
            {"foos": {"type": "list", "data": [{"id": "nested"}]}}, "key"
        )

        nested = obj.foos.data[0]

        assert obj.api_key == "key"
        assert nested.id == "nested"
        assert nested.api_key == "key"

    def test_refresh_from_nested_object(self):
        obj = telnyx.telnyx_object.TelnyxObject.construct_from(SAMPLE_FOO, "key")

        assert len(obj.calls.inbound) == 1
        assert isinstance(obj.calls.inbound[0], telnyx.telnyx_object.TelnyxObject)
        assert obj.calls.inbound[0].metadata.foo == "bar"

    def test_to_json(self):
        obj = telnyx.telnyx_object.TelnyxObject.construct_from(SAMPLE_FOO, "key")

        self.check_data(json.loads(str(obj)))

    def check_data(self, data):
        # Check rough structure
        assert len(list(data.keys())) == 7
        assert len(list(data["calls"].keys())) == 3
        assert len(data["calls"]["inbound"]) == 1
        assert len(data["calls"]["outbound"]) == 0
        assert len(data["calls"]["on_net"]) == 0

    def test_repr(self):
        obj = telnyx.telnyx_object.TelnyxObject("foo", "bar", myparam=5)

        obj["record_type"] = "\u4e00boo\u1f00"
        obj.date = datetime.datetime.fromtimestamp(1511136000)

        res = repr(obj)

        if six.PY2:
            res = six.text_type(repr(obj), "utf-8")

        assert "<TelnyxObject \u4e00boo\u1f00" in res
        assert "id=foo" in res
        assert '"date": 1511136000' in res

    def test_pickling(self):
        obj = telnyx.telnyx_object.TelnyxObject("foo", "bar", myparam=5)

        obj["object"] = "boo"
        obj.refresh_from(
            {
                "fala": "lalala",
                # ensures that empty strings are correctly unpickled in Py3
                "emptystring": "",
            },
            api_key="bar",
            partial=True,
        )

        assert obj.fala == "lalala"

        pickled = pickle.dumps(obj)
        newobj = pickle.loads(pickled)

        assert newobj.id == "foo"
        assert newobj.api_key == "bar"
        assert newobj["object"] == "boo"
        assert newobj.fala == "lalala"
        assert newobj.emptystring == ""

    def test_deletion(self):
        obj = telnyx.telnyx_object.TelnyxObject("id", "key")

        obj.coupon = "foo"
        assert obj.coupon == "foo"

        del obj.coupon
        with pytest.raises(AttributeError):
            obj.coupon

        obj.refresh_from({"coupon": "foo"}, api_key="bar", partial=True)
        assert obj.coupon == "foo"

    def test_copy(self):
        nested = telnyx.telnyx_object.TelnyxObject.construct_from(
            {"value": "bar"}, "mykey"
        )
        obj = telnyx.telnyx_object.TelnyxObject.construct_from(
            {"empty": "", "value": "foo", "nested": nested}, "mykey"
        )

        copied = copy(obj)

        assert copied.empty == ""
        assert copied.value == "foo"
        assert copied.nested.value == "bar"

        assert copied.api_key == "mykey"

        # Verify that we're not deep copying nested values.
        assert id(nested) == id(copied.nested)

    def test_deepcopy(self):
        nested = telnyx.telnyx_object.TelnyxObject.construct_from(
            {"value": "bar"}, "mykey"
        )
        obj = telnyx.telnyx_object.TelnyxObject.construct_from(
            {"empty": "", "value": "foo", "nested": nested}, "mykey"
        )

        copied = deepcopy(obj)

        assert copied.empty == ""
        assert copied.value == "foo"
        assert copied.nested.value == "bar"

        assert copied.api_key == "mykey"

        # Verify that we're actually deep copying nested values.
        assert id(nested) != id(copied.nested)

    def test_serialize_empty_string_unsets(self):
        class SerializeToEmptyString(telnyx.telnyx_object.TelnyxObject):
            def serialize(self, previous):
                return ""

        nested = SerializeToEmptyString.construct_from({"value": "bar"}, "mykey")
        obj = telnyx.telnyx_object.TelnyxObject.construct_from(
            {"nested": nested}, "mykey"
        )

        assert obj.serialize(None) == {"nested": ""}



from __future__ import absolute_import, division, print_function

import base64
import time

import pytest
from nacl.encoding import Base64Encoder
from nacl.signing import SigningKey

import telnyx

DUMMY_WEBHOOK_PAYLOAD = b"""{
    "record_type": "event",
    "id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
    "event_type": "call_initiated",
    "created_at": "2018-02-02T22:25:27.521992Z",
    "payload": {
        "to": "+13129457420",
        "start_time": "2018-02-02T22:25:27.521992Z",
        "occurred_at": "2018-02-02T22:25:27.521992Z",
        "from": "+35319605860",
        "call_control_id": "AgDIxmoRX6QMuaIj_uXRXnPAXP0QlNfXczRrZvZakpWxBlpw48KyZQ==",
        "connection_id": "7267xxxxxxxxxxxxxx",
        "call_leg_id": "428c31b6-7af4-4bcb-b7f5-5013ef9657c1",
        "call_session_id": "428c31b6-abf3-3bc1-b7f4-5013ef9657c1",
        "client_state": "aGF2ZSBhIG5pY2UgZGF5ID1d",
        "direction": "incoming",
        "state": "parked"
    }
}"""


@pytest.fixture
def signing_key():
    return SigningKey.generate()


@pytest.fixture
def signing_key_base64(signing_key):
    return signing_key.encode(encoder=Base64Encoder)


@pytest.fixture
def verify_key_base64(signing_key):
    return signing_key.verify_key.encode(encoder=Base64Encoder)


@pytest.fixture
def signer(signing_key):
    def _signer(payload, timestamp=None):
        if timestamp is None:
            timestamp = str(int(time.time())).encode("UTF-8")

        if hasattr(payload, "encode"):
            payload = payload.encode("utf-8")

        if hasattr(timestamp, "encode"):
            timestamp = timestamp.encode("utf-8")

        signed_payload = timestamp + b"|" + payload
        signed_message = signing_key.sign(signed_payload)
        signature = signed_message.signature

        return base64.b64encode(signature)

    return _signer


def test_construct_event(verify_key_base64, signer):
    telnyx.public_key = verify_key_base64

    timestamp = str(int(time.time())).encode("UTF-8")
    signature = signer(DUMMY_WEBHOOK_PAYLOAD, timestamp=timestamp)

    event = telnyx.Webhook.construct_event(DUMMY_WEBHOOK_PAYLOAD, signature, timestamp)
    assert isinstance(event, telnyx.Event)


def test_raise_on_json_error(verify_key_base64, signer):
    payload = b"this is not valid JSON"
    telnyx.public_key = verify_key_base64

    timestamp = str(int(time.time())).encode("UTF-8")
    signature = signer(payload, timestamp=timestamp)
    with pytest.raises(ValueError):
        telnyx.Webhook.construct_event(payload, signature, timestamp)


# def test_raise_on_invalid_header(verify_key_base64, signer):
#     telnyx.public_key = verify_key_base64
#     signature = "FAKEFAKEFAKE"
#     timestamp = str(int(time.time()))
#     with pytest.raises(telnyx.error.SignatureVerificationError):
#         telnyx.Webhook.construct_event(DUMMY_WEBHOOK_PAYLOAD, signature, timestamp)


def test_construct_event_from_strings(verify_key_base64, signer):
    telnyx.public_key = verify_key_base64
    timestamp = str(int(time.time()))
    signature = signer(DUMMY_WEBHOOK_PAYLOAD.decode("UTF-8"), timestamp=timestamp)
    event = telnyx.Webhook.construct_event(
        DUMMY_WEBHOOK_PAYLOAD.decode("UTF-8"), signature, timestamp
    )
    assert isinstance(event, telnyx.Event)


class TestWebhookSignature(object):
    # def test_raise_on_invalid_signature_for_payload(self, verify_key_base64, signer):
    #     telnyx.public_key = verify_key_base64
    #     timestamp = str(int(time.time())).encode('UTF-8')
    #     _signature = signer(DUMMY_WEBHOOK_PAYLOAD, timestamp)
    #     wrong_signature = signer(b"foo", timestamp)
    #     with pytest.raises(
    #         telnyx.error.SignatureVerificationError,
    #         match="Signature is invalid and does not match the payload",
    #     ):
    #         telnyx.WebhookSignature.verify(
    #             DUMMY_WEBHOOK_PAYLOAD, wrong_signature, timestamp
    #         )

    def test_raise_on_timestamp_outside_tolerance(self, verify_key_base64, signer):
        telnyx.public_key = verify_key_base64
        timestamp = str(int(time.time()) - 15).encode("UTF-8")
        signature = signer(DUMMY_WEBHOOK_PAYLOAD, timestamp)
        with pytest.raises(
            telnyx.error.SignatureVerificationError,
            match="Timestamp outside the tolerance zone",
        ):
            telnyx.WebhookSignature.verify(
                DUMMY_WEBHOOK_PAYLOAD, signature, timestamp, tolerance=10
            )

    def test_valid_header_and_signature(self, verify_key_base64, signer):
        telnyx.public_key = verify_key_base64
        timestamp = str(int(time.time())).encode("UTF-8")
        signature = signer(DUMMY_WEBHOOK_PAYLOAD, timestamp)
        assert telnyx.WebhookSignature.verify(
            DUMMY_WEBHOOK_PAYLOAD, signature, timestamp, tolerance=10
        )

    def test_timestamp_off_but_no_tolerance(self, verify_key_base64, signer):
        telnyx.public_key = verify_key_base64
        signature = signer(DUMMY_WEBHOOK_PAYLOAD, b"12345")
        assert telnyx.WebhookSignature.verify(
            DUMMY_WEBHOOK_PAYLOAD, signature, b"12345"
        )

    def test_verify_without_public_key(self, signer):
        telnyx.public_key = None
        timestamp = str(int(time.time())).encode("UTF-8")
        signature = signer(DUMMY_WEBHOOK_PAYLOAD, b"12345")
        with pytest.raises(
            telnyx.error.SignatureVerificationError, match="Public key not set"
        ):
            telnyx.WebhookSignature.verify(
                DUMMY_WEBHOOK_PAYLOAD, signature, timestamp, tolerance=10
            )



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "update_emergency_settings",
    path="/v2/phone_numbers/jobs/update_emergency_settings",
    operations=["create"],
)
@nested_resource_class_methods(
    "update_phone_numbers",
    path="/v2/phone_numbers/jobs/update_phone_numbers",
    operations=["create"],
)
@nested_resource_class_methods(
    "delete_phone_numbers",
    path="/v2/phone_numbers/jobs/delete_phone_numbers",
    operations=["create"],
)
class PhoneNumberJob(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "phone_number_job"

    @classmethod
    def class_url(cls):
        return "/v2/phone_numbers/jobs"

    @classmethod
    def update_emergency_settings(cls, **params):
        return PhoneNumberJob.create_update_emergency_settings(None, **params)

    @classmethod
    def update_phone_numbers(cls, **params):
        return PhoneNumberJob.create_update_phone_numbers(None, **params)

    @classmethod
    def delete_phone_numbers(cls, **params):
        return PhoneNumberJob.create_delete_phone_numbers(None, **params)



from __future__ import absolute_import, division, print_function

import pytest
import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


class TestOutboundVoiceProfile(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.OutboundVoiceProfile.list()
        request_mock.assert_requested("get", "/v2/outbound_voice_profiles")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.OutboundVoiceProfile)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_retrievable(self, request_mock):
        resource = telnyx.OutboundVoiceProfile.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/outbound_voice_profiles/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.OutboundVoiceProfile)

    def test_is_creatable(self, request_mock):
        resource = telnyx.OutboundVoiceProfile.create(name="my-profile")
        request_mock.assert_requested("post", "/v2/outbound_voice_profiles")
        assert isinstance(resource, telnyx.OutboundVoiceProfile)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_saveable(self, request_mock):
        outbound_voice_profile = telnyx.OutboundVoiceProfile.retrieve(TEST_RESOURCE_ID)
        outbound_voice_profile.concurrent_call_limit = 10
        outbound_voice_profile.name = "Dan"
        resource = outbound_voice_profile.save()
        request_mock.assert_requested(
            "patch", "/v2/outbound_voice_profiles/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.OutboundVoiceProfile)
        assert resource is outbound_voice_profile

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_modifiable(self, request_mock):
        resource = telnyx.OutboundVoiceProfile.modify(
            TEST_RESOURCE_ID, concurrent_call_limit=10, name="Dan"
        )
        request_mock.assert_requested(
            "patch", "/v2/outbound_voice_profiles/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.OutboundVoiceProfile)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_deletable(self, request_mock):
        resource = telnyx.OutboundVoiceProfile.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/outbound_voice_profiles/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import ListableAPIResource


class Connection(ListableAPIResource):
    OBJECT_NAME = "connection"



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


@pytest.mark.skip(reason="Not implemented yet")
class TestCustomStorageCredential(object):
    def test_is_retrievable(self, request_mock):
        resource = telnyx.CustomStorageCredential.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/custom_storage_credentials/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.CustomStorageCredential)

    def test_is_creatable(self, request_mock):
        resource = telnyx.CustomStorageCredential.create(
            backend="gcs", configuration=[]
        )
        request_mock.assert_requested("post", "/v2/custom_storage_credentials")
        assert isinstance(resource, telnyx.CustomStorageCredential)

    def test_is_modifiable(self, request_mock):
        resource = telnyx.CustomStorageCredential.modify(
            TEST_RESOURCE_ID,
            active=False,
            webhook_event_url="https://update.com",
            application_name="updated name",
        )
        request_mock.assert_requested(
            "patch", "/v2/custom_application/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.FaxApplication)

    def test_is_deletable(self, request_mock):
        resource = telnyx.CustomStorageCredential.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/custom_storage_credentials/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "sessions", path="sessions", operations=["list", "retrieve"]
)
@nested_resource_class_methods(
    "generate_token", path="actions/generate_join_client_token", operations=["create"]
)
@nested_resource_class_methods(
    "refresh_token", path="actions/refresh_client_token", operations=["create"]
)
class Room(
    CreateableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "room"

    def list_sessions(self, **params):
        return Room.list_sessions(self.name, **params)

    def generate_token(self, **params):
        return Room.create_generate_token(self.id, **params)

    def refresh_token(self, **params):
        return Room.create_refresh_token(self.id, **params)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import APIResource, nested_resource_class_methods


@nested_resource_class_methods(
    "queue_call", path="calls", operations=["list", "retrieve"]
)
class Queue(APIResource):
    OBJECT_NAME = "queue"

    def list_calls(self, **params):
        return Queue.list_queue_calls(self.name, **params)

    def retrieve_call(self, call_control_id, **params):
        return Queue.retrieve_queue_call(self.name, call_control_id, **params)


class QueueCall(APIResource):
    OBJECT_NAME = "queue_call"



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract.api_resource import APIResource
from telnyx.api_resources.abstract.createable_api_resource import CreateableAPIResource
from telnyx.api_resources.abstract.deletable_api_resource import DeletableAPIResource
from telnyx.api_resources.abstract.listable_api_resource import ListableAPIResource
from telnyx.api_resources.abstract.nested_resource_class_methods import (
    nested_resource_class_methods,
)
from telnyx.api_resources.abstract.singleton_api_resource import SingletonAPIResource
from telnyx.api_resources.abstract.updateable_api_resource import UpdateableAPIResource

# flake8: noqa



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
)


class NotificationSetting(
    ListableAPIResource, CreateableAPIResource, DeletableAPIResource
):
    OBJECT_NAME = "notification_setting"



from __future__ import absolute_import, division, print_function

import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


class TestPhoneNumberJob(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.PhoneNumberJob.list()
        request_mock.assert_requested("get", "/v2/phone_numbers/jobs")
        assert isinstance(resources.data, list)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.PhoneNumberJob.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/phone_numbers/jobs/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.PhoneNumberJob)

    def test_update_emergency_settings(self, request_mock):
        telnyx.PhoneNumberJob.update_emergency_settings(
            emergency_address_id="53829456729313",
            emergency_enabled=True,
            phone_numbers=[],
        )
        request_mock.assert_requested(
            "post", "/v2/phone_numbers/jobs/update_emergency_settings"
        )

    def test_update_phone_numbers(self, request_mock):
        telnyx.PhoneNumberJob.update_phone_numbers(phone_numbers=[])
        request_mock.assert_requested(
            "post", "/v2/phone_numbers/jobs/update_phone_numbers"
        )

    def test_delete_phone_numbers(self, request_mock):
        telnyx.PhoneNumberJob.delete_phone_numbers(phone_numbers=[])
        request_mock.assert_requested(
            "post", "/v2/phone_numbers/jobs/delete_phone_numbers"
        )



import pytest

import telnyx

TEST_RESOURCE_ID = "12ade33a-21c0-473b-b055-b3c836e1c292"


@pytest.mark.skip(reason="Updated_Endpoint")
class TestVerifyProfile(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.VerifyProfile.list()
        request_mock.assert_requested("get", "/v2/verify_profiles")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.VerifyProfile)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.VerifyProfile.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/verify_profiles/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.VerifyProfile)

    def test_is_saveable(self, request_mock):
        resource = telnyx.VerifyProfile.create(name="hi there")
        request_mock.assert_requested("post", "/v2/verify_profiles")
        assert isinstance(resource, telnyx.VerifyProfile)
        assert resource is resource

    def test_is_modifiable(self, request_mock):
        _ = telnyx.VerifyProfile.modify(TEST_RESOURCE_ID, name="hi there")
        request_mock.assert_requested(
            "patch", "/v2/verify_profiles/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "45f45a04-b4be-4592-95b1-9306b9db2b21"


@pytest.mark.skip(reason="Filter looking for 'filter%5bphone_number%5d', typo")
class TestPhoneNumberRegulatoryRequirement(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.PhoneNumberRegulatoryRequirement.list(
            filter={"phone_number": "+41215470622,+41215470633"}
        )
        request_mock.assert_requested(
            "get", "/v2/phone_numbers_regulatory_requirements"
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.PhoneNumberRegulatoryRequirement)

    def test_is_retrievable(self, request_mock):
        with pytest.raises(telnyx.error.InvalidRequestError):
            telnyx.PhoneNumberRegulatoryRequirement.retrieve(TEST_RESOURCE_ID)



from __future__ import absolute_import, division, print_function

import telnyx


class TestCreateableAPIResource(object):
    class MyCreatable(telnyx.api_resources.abstract.CreateableAPIResource):
        OBJECT_NAME = "mycreatable"

    def test_create(self, request_mock):
        request_mock.stub_request(
            "post",
            "/v2/mycreatables",
            {"data": {"record_type": "messaging_profile", "foo": "bar"}},
            rheaders={"request-id": "req_id"},
        )

        res = self.MyCreatable.create()

        request_mock.assert_requested("post", "/v2/mycreatables", {})
        assert isinstance(res, telnyx.MessagingProfile)
        assert res.foo == "bar"

        assert res.last_response is not None
        assert res.last_response.request_id == "req_id"

    def test_idempotent_create(self, request_mock):
        request_mock.stub_request(
            "post",
            "/v2/mycreatables",
            {"data": {"record_type": "messaging_profile", "foo": "bar"}},
        )

        res = self.MyCreatable.create()

        request_mock.assert_requested("post", "/v2/mycreatables", {}, {})
        assert isinstance(res, telnyx.MessagingProfile)
        assert res.foo == "bar"



from __future__ import absolute_import, division, print_function

import telnyx

TEST_RESOURCE_ID = "3fa85f64-5717-4562-b3fc-2c963f66afa6"


class TestShortCode(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.ShortCode.list()
        request_mock.assert_requested("get", "/v2/short_codes")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.ShortCode)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.ShortCode.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/short_codes/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.ShortCode)

    def test_is_saveable(self, request_mock):
        short_code = telnyx.ShortCode.retrieve(TEST_RESOURCE_ID)
        short_code.messaging_profile_id = "2"
        resource = short_code.save()
        request_mock.assert_requested("patch", "/v2/short_codes/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.ShortCode)
        assert resource is short_code

    def test_is_modifiable(self, request_mock):
        resource = telnyx.ShortCode.modify(TEST_RESOURCE_ID, messaging_profile_id="2")
        request_mock.assert_requested("patch", "/v2/short_codes/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.ShortCode)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "phone_numbers", path="phoneNumbers", operations=["list"]
)
class PhoneNumberByProfile(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "phone_number_assignment_by_profile"

    @classmethod
    def class_url(cls):
        return "/10dlc/phoneNumberAssignmentByProfile"

    def phone_numbers(self, **params):
        return PhoneNumberByProfile.list_mno_metadata(self.name, **params)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import six

from telnyx import error


class TestTelnyxError(object):
    def test_formatting(self):
        err = error.TelnyxError("öre")
        assert six.text_type(err) == "öre"
        if six.PY2:
            assert str(err) == "\xc3\xb6re"
        else:
            assert str(err) == "öre"

    def test_formatting_with_request_id(self):
        err = error.TelnyxError("öre", http_headers={"request-id": "123"})
        assert six.text_type(err) == "Request 123: öre"
        if six.PY2:
            assert str(err) == "Request 123: \xc3\xb6re"
        else:
            assert str(err) == "Request 123: öre"

    def test_formatting_with_none(self):
        err = error.TelnyxError(None, http_headers={"request-id": "123"})
        assert six.text_type(err) == "Request 123: <empty message>"
        if six.PY2:
            assert str(err) == "Request 123: <empty message>"
        else:
            assert str(err) == "Request 123: <empty message>"

    def test_formatting_with_message_none_and_request_id_none(self):
        err = error.TelnyxError(None)
        assert six.text_type(err) == "<empty message>"
        if six.PY2:
            assert str(err) == "<empty message>"
        else:
            assert str(err) == "<empty message>"

    def test_repr(self):
        err = error.TelnyxError("öre", http_headers={"request-id": "123"})
        if six.PY2:
            assert (
                repr(err) == "TelnyxError(message=u'\\xf6re', http_status=None, "
                "request_id='123')"
            )
        else:
            assert (
                repr(err) == "TelnyxError(message='öre', http_status=None, "
                "request_id='123')"
            )


class TestApiConnectionError(object):
    def test_default_no_retry(self):
        err = error.APIConnectionError("msg")
        assert err.should_retry is False

        err = error.APIConnectionError("msg", should_retry=True)
        assert err.should_retry



from __future__ import absolute_import, division, print_function

import telnyx

TEST_RESOURCE_ID = "f1486bae-f067-460c-ad43-73a92848f902"


class TestPortOut(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.PortOut.list()
        request_mock.assert_requested("get", "/v2/portouts")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.PortOut)

    def test_is_retrievable(self, request_mock):
        resource = telnyx.PortOut.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/portouts/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.PortOut)

    def test_portout_comments_is_listable(self, request_mock):
        resource = telnyx.PortOut()
        resource.list_comments(id=TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/portouts/%s/comments" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.PortOut)

    def test_portout_comments_is_createable(self, request_mock):
        resource = telnyx.PortOut.retrieve(TEST_RESOURCE_ID)
        resource.create_comments(id=TEST_RESOURCE_ID, body="This is a body")
        request_mock.assert_requested(
            "post", "/v2/portouts/%s/comments" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.PortOut)

    def test_portout_support_documents_is_listable(self, request_mock):
        resource = telnyx.PortOut()
        resource.list_supporting_documents(id=TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/portouts/%s/supporting_documents" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.PortOut)

    def test_portout_support_documents_is_createable(self, request_mock):
        resource = telnyx.PortOut.retrieve(TEST_RESOURCE_ID)
        resource.create_supporting_documents(id=TEST_RESOURCE_ID, body="This is a body")
        request_mock.assert_requested(
            "post", "/v2/portouts/%s/supporting_documents" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.PortOut)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
)


class PublicInternetGateway(
    CreateableAPIResource, DeletableAPIResource, ListableAPIResource
):
    OBJECT_NAME = "public_internet_gateway"



import os
import sys
from codecs import open

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [("pytest-args=", "a", "Arguments to pass into pytest")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = "-n auto"

    def run_tests(self):
        import shlex

        import pytest

        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


here = os.path.abspath(os.path.dirname(__file__))

os.chdir(here)

with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="telnyx",
    version="2.1.4",
    description="Python bindings for the Telnyx API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Telnyx",
    author_email="support@telnyx.com",
    url="https://github.com/team-telnyx/telnyx-python",
    project_urls={
        "Documentation": "https://developers.telnyx.com/docs/api/v2/overview",
        "Source Code": "https://github.com/team-telnyx/telnyx-python",
        "Bug Tracker": "https://github.com/team-telnyx/telnyx-python/issues",
    },
    license="MIT",
    keywords="telnyx telephony sip networking callcontrol messaging sms mms",
    packages=find_packages(exclude=["tests", "tests.*"]),
    zip_safe=False,
    install_requires=[
        "requests >= 2.20",
        "six >= 1.16.0",
        "PyNaCl"
    ],
    python_requires=">=3.8",
    tests_require=[
        "pytest >= 4",
        "pytest-mock >= 1.7",
        "pytest-xdist >= 1.22",
        "pytest-cov >= 2.5",
    ],
    cmdclass={"test": PyTest},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Communications :: Telephony",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



from __future__ import absolute_import, division, print_function

import pytest
import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


class TestFaxApplication(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.FaxApplication.list()
        request_mock.assert_requested("get", "/v2/fax_applications")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.FaxApplication)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_retrievable(self, request_mock):
        resource = telnyx.FaxApplication.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v2/fax_applications/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.FaxApplication)

    def test_is_creatable(self, request_mock):
        resource = telnyx.FaxApplication.create(
            active=True,
            application_name="Test Name",
            webhook_event_url="https://test.com",
        )
        request_mock.assert_requested("post", "/v2/fax_applications")
        assert isinstance(resource, telnyx.FaxApplication)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_saveable(self, request_mock):
        fax_application = telnyx.FaxApplication.retrieve(TEST_RESOURCE_ID)
        fax_application.active = False
        fax_application.webhook_event_url = "https://update.com"
        fax_application.application_name = "updated name"
        resource = fax_application.save()
        request_mock.assert_requested(
            "patch", "/v2/fax_applications/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.FaxApplication)
        assert resource is fax_application

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_modifiable(self, request_mock):
        resource = telnyx.FaxApplication.modify(
            TEST_RESOURCE_ID,
            active=False,
            webhook_event_url="https://update.com",
            application_name="updated name",
        )
        request_mock.assert_requested(
            "patch", "/v2/fax_applications/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, telnyx.FaxApplication)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_deletable(self, request_mock):
        resource = telnyx.FaxApplication.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/fax_applications/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods("enable", path="actions/enable", operations=["create"])
@nested_resource_class_methods("disable", path="actions/disable", operations=["create"])
class ManagedAccount(CreateableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME = "managed_account"

    def enable(self, **params):
        return self.create_enable(**params)

    def disable(self, **params):
        return self.create_disable(**params)



from __future__ import absolute_import, division, print_function

import telnyx

TEST_MESSAGING_PROFILE_ID = "d120432d-2e77-4583-87f8-1db837cee559"
TEST_SRC_LONG_CODE = "+13125550100"
TEST_SRC_ALPHANUMERIC = "Testing 123"
TEST_DST = "+17735550100"
TEST_MESSAGE_BODY = "Hello!"


class TestMessage(object):
    def test_can_create_standard_message(self, request_mock):
        resource = telnyx.Message.create(
            from_=TEST_SRC_LONG_CODE, to=TEST_DST, text=TEST_MESSAGE_BODY
        )
        request_mock.assert_requested("post", "/v2/messages")
        assert isinstance(resource, telnyx.Message)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods("disable", path="actions/disable", operations=["create"])
@nested_resource_class_methods(
    "delete_network_preferences",
    path="actions/delete_network_preferences",
    operations=["create"],
)
@nested_resource_class_methods(
    "bulk_set_network_preferences",
    path="actions/bulk_set_network_preferences",
    operations=["create"],
)
@nested_resource_class_methods(
    "validate_registration_codes",
    path="actions/validate_registration_codes",
    operations=["create"],
)
@nested_resource_class_methods(
    "set_public_ip", path="actions/set_public_ip", operations=["create"]
)
@nested_resource_class_methods(
    "set_network_preferences",
    path="actions/set_network_preferences",
    operations=["create"],
)
@nested_resource_class_methods("enable", path="actions/enable", operations=["create"])
@nested_resource_class_methods(
    "set_standby", path="actions/set_standby", operations=["create"]
)
@nested_resource_class_methods(
    "remove_public_ip", path="actions/remove_public_ip", operations=["create"]
)
@nested_resource_class_methods(
    "bulk_set_public_ips", path="actions/bulk_set_public_ips", operations=["create"]
)
@nested_resource_class_methods(
    "register", path="/v2/actions/register/sim_cards", operations=["create"]
)
class SIMCard(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "sim_card"

    def disable(self, **params):
        return self.create_disable(**params)

    def delete_network_preferences(self, **params):
        return self.create_delete_network_preferences(**params)

    def bulk_set_network_preferences(self, **params):
        return self.create_bulk_set_network_preferences(**params)

    def validate_registration_codes(self, **params):
        return self.create_validate_registration_codes(**params)

    def set_public_ip(self, **params):
        return self.create_set_public_ip(**params)

    def set_network_preferences(self, **params):
        return self.create_set_network_preferences(**params)

    def enable(self, **params):
        return self.create_enable(**params)

    def set_standby(self, **params):
        return self.create_set_standby(**params)

    def remove_public_ip(self, **params):
        return self.create_remove_public_ip(**params)

    def bulk_set_public_ips(self, **params):
        return self.create_bulk_set_public_ips(**params)

    @classmethod
    def register(cls, **params):
        return SIMCard.create_register(None, **params)



from __future__ import absolute_import, division, print_function

from telnyx import api_requestor, util
from telnyx.api_resources.abstract.api_resource import APIResource


class ListableAPIResource(APIResource):
    @classmethod
    def auto_paging_iter(cls, *args, **params):
        return cls.list(*args, **params).auto_paging_iter()

    @classmethod
    def auto_paging_iter_by_token(cls, *args, **params):
        return cls.list(*args, **params).auto_paging_iter_by_token()

    @classmethod
    def list(cls, api_key=None, **params):
        requestor = api_requestor.APIRequestor(api_key, api_base=cls.api_base())
        url = cls.class_url()
        response, api_key = requestor.request("get", url, params)
        response.data["url"] = url

        telnyx_object = util.convert_to_telnyx_object(response, api_key)
        telnyx_object._retrieve_params = params
        return telnyx_object



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "detailed_status", path="detailedStatus", operations=["list"]
)
class BulkCreation(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "bulk_creation"

    @classmethod
    def class_url(cls):
        return "/10dlc/bulkCreation"

    def detailed_status(self, **params):
        return BulkCreation.list_detailed_status(self.name, **params)



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    ListableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods("extend", path="actions/extend", operations=["create"])
class PhoneNumberReservation(CreateableAPIResource, ListableAPIResource):
    OBJECT_NAME = "phone_number_reservation"

    def extend(self, **params):
        return self.create_extend(**params)



from __future__ import absolute_import, division, print_function

import telnyx


class TestAvailablePhoneNumber(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.AvailablePhoneNumber.list(
            filter={
                "limit": 1,
                "features": ["sms", "mms"],
                "phone_number": {"contains": "555"},
            }
        )
        request_mock.assert_requested("get", "/v2/available_phone_numbers")
        assert isinstance(resources.data, list)
        assert len(resources.data) == 1
        assert isinstance(resources.data[0], telnyx.AvailablePhoneNumber)



from __future__ import absolute_import, division, print_function

import pytest
import telnyx

TEST_RESOURCE_ID = "1293384261075731499"


class TestFQDN(object):
    def test_is_listable(self, request_mock):
        resources = telnyx.FQDN.list()
        request_mock.assert_requested("get", "/v2/fqdns")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], telnyx.FQDN)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_retrievable(self, request_mock):
        resource = telnyx.FQDN.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v2/fqdns/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.FQDN)

    def test_is_creatable(self, request_mock):
        resource = telnyx.FQDN.create(
            fqdn="example.com", dns_record_type="a", connection_id="1123"
        )
        request_mock.assert_requested("post", "/v2/fqdns")
        assert isinstance(resource, telnyx.FQDN)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_saveable(self, request_mock):
        fqdn = telnyx.FQDN.retrieve(TEST_RESOURCE_ID)
        fqdn.fqdn = "new-example.com"
        resource = fqdn.save()
        request_mock.assert_requested("patch", "/v2/fqdns/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.FQDN)
        assert resource is fqdn

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_modifiable(self, request_mock):
        resource = telnyx.FQDN.modify(TEST_RESOURCE_ID, fqdn="new-example.com")
        request_mock.assert_requested("patch", "/v2/fqdns/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, telnyx.FQDN)

    @pytest.mark.skip(reason="Prism mock 500 invalid response")
    def test_is_deletable(self, request_mock):
        resource = telnyx.FQDN.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested("delete", "/v2/fqdns/%s" % TEST_RESOURCE_ID)



from __future__ import absolute_import, division, print_function

import six
from six.moves.urllib.parse import quote_plus

from telnyx import error, util
from telnyx.api_resources.abstract import (
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods(
    "voice", operations=["list", "update"], pluralize_path=False
)
@nested_resource_class_methods(
    "enable_emergency", path="actions/enable_emergency", operations=["create"]
)
@nested_resource_class_methods(
    "messaging", path="messaging", operations=["list", "update"], pluralize_path=False
)
class PhoneNumber(DeletableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME = "phone_number"

    @classmethod
    def all_voice(cls, **params):
        """Returns the voice settings for /all/ numbers owned by the user.

        This method breaks the naming convention of helper methods by adding
        an `all_` prefix, which might be confusing at first. The reason for
        this prefix is the `voice()` method name is already taken. The
        Telnyx API supports these two endpoints:

        - /v2/phone_numbers/{id}/voice
        - /v2/phone_numbers/voice

        The `/{id}/voice` endpoint is taken by the instance method `voice()`.
        As we can't nicely re-use this method for two different endpoints,
        we have this `all_voice()` endpoint to support the `/voice` endpoint.
        """

        return PhoneNumber.list_voice(None, **params)

    def voice(self, **params):
        """Returns the voice settings for the instantiated phone number."""

        return PhoneNumber.list_voice(self.id, **params)

    def enable_emergency(self, **params):
        return PhoneNumber.create_enable_emergency(self.id, **params)

    @classmethod
    def all_messaging(cls, **params):
        """Returns the messaging settings for /all/ numbers owned by the
        user.

        See the documentation for `all_voice()` for an explanation on the
        difference between `all_messaging()` and `messaging()`.
        """

        return PhoneNumber.list_messaging(None, **params)

    def messaging(self, **params):
        """Returns the messaging settings for the instantiated phone
        number.
        """

        return PhoneNumber.list_messaging(self.id, **params)


class VoiceSettings(ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME = "voice_settings"

    @classmethod
    def class_url(cls):
        return "/v2/phone_numbers/voice"

    def instance_url(self):
        id = self.get("id")

        if not isinstance(id, six.string_types):
            raise error.InvalidRequestError(
                "Could not determine which URL to request: %s instance "
                "has invalid ID: %r, %s. ID should be of type `str` (or"
                " `unicode`)" % (type(self).__name__, id, type(id)),
                "id",
            )

        id = util.utf8(id)
        extn = quote_plus(id)
        return "/v2/phone_numbers/%s/voice" % extn

    @classmethod
    def modify(cls, sid, **params):
        url = "/v2/phone_numbers/%s/voice" % quote_plus(util.utf8(sid))
        return cls._modify(url, **params)


class MessagingSettings(ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME = "messaging_settings"

    @classmethod
    def class_url(cls):
        return "/v2/phone_numbers/messaging"

    def instance_url(self):
        id = self.get("id")

        if not isinstance(id, six.string_types):
            raise error.InvalidRequestError(
                "Could not determine which URL to request: %s instance "
                "has invalid ID: %r, %s. ID should be of type `str` (or"
                " `unicode`)" % (type(self).__name__, id, type(id)),
                "id",
            )

        id = util.utf8(id)
        extn = quote_plus(id)
        return "/v2/phone_numbers/%s/messaging" % extn

    @classmethod
    def modify(cls, sid, **params):
        url = "/v2/phone_numbers/%s/messaging" % quote_plus(util.utf8(sid))
        return cls._modify(url, **params)



from __future__ import absolute_import, division, print_function

import pytest

import telnyx

TEST_RESOURCE_ID = "6a09cdc3-8948-47f0-aa62-74ac943d6c58"


class TestMessagingHostedNumber(object):
    @pytest.mark.skip(reason="Retrieve resource no implemented")
    def test_is_deletable(self, request_mock):
        resource = telnyx.MessagingHostedNumber()
        resource.id = TEST_RESOURCE_ID
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v2/messaging_hosted_numbers/%s" % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

from telnyx.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
    nested_resource_class_methods,
)


@nested_resource_class_methods("confirm", path="actions/confirm", operations=["create"])
@nested_resource_class_methods("cancel", path="actions/cancel", operations=["create"])
@nested_resource_class_methods(
    "loa_template", path="loa_template", operations=["retrieve"]
)
@nested_resource_class_methods(
    "allowed_foc_windows", path="allowed_foc_windows", operations=["list"]
)
@nested_resource_class_methods(
    "activation_jobs", path="activation_jobs", operations=["list"]
)
class PortingOrder(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "porting_order"

    def confirm(self, **params):
        return PortingOrder.create_confirm(self.id, **params)

    def loaTemplate(self, **params):
        return PortingOrder.retrieve_loa_template(self.id, **params)

    def cancel(self, **params):
        return PortingOrder.create_cancel(self.id, **params)

    def allowed_foc_windows(self, **params):
        return PortingOrder.list_allowed_foc_windows(self.id, **params)

    def activation_jobs(self, **params):
        return PortingOrder.list_activation_jobs(self.id, **params)


class PortingPhoneNumber(ListableAPIResource):
    OBJECT_NAME = "porting_phone_number"
